package com.aza.vp.game;

import java.util.HashMap;
import java.util.Map;


public abstract class Poker
{
    public int getMaxBet()
    {
        return MAX_BET;
    }   
    
    public static final int MAX_BET = 5;
    
	public static String[] card_sym = {"2", "3", "4", "5", "6", "7", "8", "9", "10", "J", "Q", "K", "A"};
    public static String card_colors[] = {"C", "D", "H", "S"};

    public static String getCardString(int c)
    {
    	if (c <= 51)
    		return card_sym[(c % 13)] + card_colors[(c / 13)];
    	else
    		return "W";
    }
    
    public static void print_hand(int hand[])
    {
    	System.out.println();
    	for (int i = 0; i < hand.length; i++)
    	{
    		System.out.print(getCardString(hand[i]) + ", ");
		}
    	System.out.println();
    }
    
    public static final int NOTHING = 100;
    
	public abstract int[] getPayoutTable();

	public abstract String[] getHandCat();
	
	public abstract int getPayout(int hand);

	public abstract String getHandString(int res);

    public abstract int checkHand(int hand[], int n_wild);

    public abstract int getNumCards();

    public abstract int[] getHold();
    
    public abstract int getWildCardRank();
    
    
   
    int type;
    
    public Poker(int type)
    {
    	this.type = type;
    }
    
    public int getType()
    {
    	return type;
    }    
    
    
    public void holdAll(int hold[])
    {
        for (int i = 0; i < hold.length; i++)
        {
            hold[i] = 1;
        }
    }

    public void holdCard(int hand[], int card, int hold[])
    {
        for (int i = 0; i < hold.length; i++)
        {
            if (hand[i] % 13 == card)
                hold[i] = 1;
        }
    }

    public void holdHighCard(int hand[], int hold[], int min)
    {    	
        int m = -1;
        int c = min;
        for (int i = 0; i < hold.length; i++)
        {
            if ((hand[i] % 13) >= c)
            {
                m = i;
                c = hand[i] % 13;
            }
        }
        
        if (m > -1)
        hold[m] = 1;
    }
    
    public static int wild_count(int hold[], int wild_rank)
    {
    	int n_wild = 0;
    	for (int i = 0; i < hold.length; i++)
		{
			if (hold[i] % 13 == wild_rank || hold[i] > 51) n_wild++;
		}
    	
    	return n_wild;
    }    
    
    public static final int TYPE_JACKS_OR_BETTER = 0;
    public static final int TYPE_DEUCES_WILD = 1;
    public static final int TYPE_TENS_OR_BETTER = 2;
    public static final int TYPE_JOKER_POKER = 3;
    public static final int TYPE_DOUBLE_BONUS = 4;    
    
    public static Poker createGame(int type)
    {
    	Poker poker = null;
    	if (type == TYPE_DEUCES_WILD)
    	{
    		poker = new DeucesWild(type);
    	}
    	else if (type == TYPE_JACKS_OR_BETTER)
    	{
    		poker = new JacksOrBetter(type);
    	}
    	else if (type == TYPE_TENS_OR_BETTER)
    	{
    		poker = new TensOrBetter(type);
    	}
    	else if (type == TYPE_DOUBLE_BONUS)
    	{
    		poker = new DoubleBonusPoker(type);
    	}
    	else if (type == TYPE_JOKER_POKER)
    	{
    		poker = new JokerPoker(type);
    	}
    	
    	return poker;
    }
    
    
    
    
    
    
    
    
    
    
    
    public static Map<String, Integer> map = new HashMap<String, Integer>();
    static
    {
    	int i = 0;
    	map.put("2C", i++);
    	map.put("3C", i++);
    	map.put("4C", i++);
    	map.put("5C", i++);
    	map.put("6C", i++);
    	map.put("7C", i++);
    	map.put("8C", i++);
    	map.put("9C", i++);
    	map.put("10C", i++);
    	map.put("JC", i++);
    	map.put("QC", i++);
    	map.put("KC", i++);
    	map.put("AC", i++);
    	
    	map.put("2D", i++);
    	map.put("3D", i++);
    	map.put("4D", i++);
    	map.put("5D", i++);
    	map.put("6D", i++);
    	map.put("7D", i++);
    	map.put("8D", i++);
    	map.put("9D", i++);
    	map.put("10D", i++);
    	map.put("JD", i++);
    	map.put("QD", i++);
    	map.put("KD", i++);
    	map.put("AD", i++);
    	
    	map.put("2H", i++);
    	map.put("3H", i++);
    	map.put("4H", i++);
    	map.put("5H", i++);
    	map.put("6H", i++);
    	map.put("7H", i++);
    	map.put("8H", i++);
    	map.put("9H", i++);
    	map.put("10H", i++);
    	map.put("JH", i++);
    	map.put("QH", i++);
    	map.put("KH", i++);
    	map.put("AH", i++);
    	
    	map.put("2S", i++);
    	map.put("3S", i++);
    	map.put("4S", i++);
    	map.put("5S", i++);
    	map.put("6S", i++);
    	map.put("7S", i++);
    	map.put("8S", i++);
    	map.put("9S", i++);
    	map.put("10S", i++);
    	map.put("JS", i++);
    	map.put("QS", i++);
    	map.put("KS", i++);
    	map.put("AS", i++);	    	
    }
    
    
    public static int[] getHand(String c1, String c2, String c3, String c4, String c5)
    {
    	int hand[] = new int[5];
    	hand[0] = map.get(c1);
    	hand[1] = map.get(c2);
    	hand[2] = map.get(c3);
    	hand[3] = map.get(c4);
    	hand[4] = map.get(c5);
    	
    	return hand;
    }
    
    
    
}
